using System;
using System.Data;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using System.ComponentModel;


namespace CTI.Forms
{
    
    public class ComboBoxGridFilterBar : GridFilterBar 
    {

        public object[] savedValues = null;

        public override void UnwireGrid()
        {
            GridDataBoundGrid grid = this.GetGrid();
            if (grid != null)
            {
                GridBoundColumnsCollection gbcc;
                gbcc = (grid.GridBoundColumns.Count == 0) ? grid.Binder.InternalColumns : grid.GridBoundColumns;
                
                savedValues = new object[gbcc.Count];

                for(int gbcIndex = 0; gbcIndex < gbcc.Count ; ++gbcIndex)
                {
                    GridStyleInfo cellStyle = gbcc[gbcIndex].StyleInfo;

                    if(cellStyle.CellType == "ComboBox" && cellStyle.DataSource != null &&
                        cellStyle.DisplayMember != cellStyle.ValueMember)
                    {
                        string colName = gbcc[gbcIndex].MappingName;
                        int col = grid.Binder.NameToColIndex(colName);
                        int row = GetFilterRow();

                        savedValues[gbcIndex] = grid[row, col].CellValue;
                        grid[row, col].DataSource = null;
                        grid[row, col].CellValue = "";

                    }
                }
            }

            base.UnwireGrid ();
        }

        
        public override void WireGrid(Syncfusion.Windows.Forms.Grid.GridDataBoundGrid grid, Syncfusion.Windows.Forms.Grid.GridStyleInfo style)
        {
            base.WireGrid(grid, style);
            
    
            //swapout the filter droplists for any combobox col with displaymember != valuemember
            // replace the default simgle item dropdown with a dropdown that handles both display and values
            GridBoundColumnsCollection gbcc;
            gbcc = (grid.GridBoundColumns.Count == 0) ? grid.Binder.InternalColumns : grid.GridBoundColumns;
                
            for(int gbcIndex = 0; gbcIndex < gbcc.Count ; ++gbcIndex)
            {
                GridStyleInfo cellStyle = gbcc[gbcIndex].StyleInfo;

                if(cellStyle.CellType == "ComboBox" && cellStyle.DataSource != null &&
                    cellStyle.DisplayMember != cellStyle.ValueMember)
                {
                    string colName = gbcc[gbcIndex].MappingName;
                    int col = grid.Binder.NameToColIndex(colName);
                    int row = GetFilterRow();

                    DataView dv = null;

                    if(cellStyle.DataSource is DataTable)
                    {
                        DataTable dt = (DataTable)cellStyle.DataSource;
                        dv = new DataView(dt, "", "", DataViewRowState.CurrentRows);
                    }
                    else if(style.DataSource is DataView)
                    {
                        dv = (DataView) cellStyle.DataSource;
                    }
                    if(dv != null)
                    {
                        dv.Sort = cellStyle.DisplayMember;
                        grid[row, col].DataSource = CreateEntries(dv, cellStyle.DisplayMember, cellStyle.ValueMember);
                        grid[row, col].DisplayMember = cellStyle.DisplayMember;
                        grid[row, col].ValueMember = cellStyle.ValueMember;
                        if (savedValues != null && savedValues.Length == gbcc.Count)
                            grid[row, col].CellValue = savedValues[gbcIndex];                   }
                        else
                            grid[row, col].CellValue = -2;
                }
            }
        }

    
        protected override string GetFilterFromRow(GridDataBoundGrid grid)
        {
            string filter = base.GetFilterFromRow(grid);

            //swap display/value strings in the default filter

            GridCurrentCell cc = grid.CurrentCell;
            GridBoundColumnsCollection gbcc;
            int row = this.GetFilterRow();

            gbcc = (grid.GridBoundColumns.Count == 0) ? grid.Binder.InternalColumns : grid.GridBoundColumns;
            // int col = cc.ColIndex;
            for(int col = 1; col <= grid.Model.ColCount; ++col)
            {
                int field = grid.Binder.ColIndexToField(col);
                if (field == -1)
                    continue;
                GridStyleInfo style = gbcc[field].StyleInfo;
                if(style.CellType == "ComboBox" && style.DataSource != null &&
                    style.DisplayMember != style.ValueMember)
                {
                    string s = "";
                    string s1 = "";
                    
                    if(cc.ColIndex == col)
                    {
                        s = "'" + cc.Renderer.ControlText + "'";
                        s1 = "'" + cc.Renderer.ControlValue.ToString() + "'" ;
                        // Save the value
                        grid[row, col].CellValue = cc.Renderer.ControlValue;
                    }
                    else
                    {
                        s = "'" + grid[row, col].GetFormattedText(grid[row, col].CellValue) + "'";
                        s1 = "'" + grid[row, col].CellValue.ToString() + "'" ;
                    
                    }
                    if (s != "''" && s1 != "''")
                        filter = filter.Replace(s, s1);
                    
                }
            }
            return filter;
        }


        //create a dropdown with both display & value members
        protected DataTable CreateEntries(DataView dv, string displayName, string valueName)
        {
            DataTable dt = new DataTable(dv.Table.TableName);
            dt.Columns.Add(new DataColumn(displayName));
            dt.Columns.Add(new DataColumn(valueName));
            DataRow dr = dt.NewRow();
            dr[0] = "(none)";
            dr[1] = "-2";
            dt.Rows.Add(dr);

            string s = "";
            for(int i = 0; i < dv.Count; ++i)
            {
                if(s != dv[i].Row[displayName].ToString())
                {
                    s = dv[i].Row[displayName].ToString();
                    dr = dt.NewRow();
                    dr[0] = s;
                    dr[1] = dv[i].Row[valueName].ToString();
                    dt.Rows.Add(dr);
                }
            }
            return dt;
        }
    }




}
